# frozen_string_literal: true

# This is exceptionally fragile, especially when considering the constant breaking
# changes from the `pip` team.

module PoisePython
  module Resources
    # Monkeypatch the deprecated poise-python into working on new versions of Pip.
    module PythonPackage
      PIP_HACK_SCRIPT = <<-HACK.gsub(/^ {8}/, '')
        import json
        import re
        import sys
        from pip import __version__ as pip_version
        # Don't use pkg_resources because I don't want to require it before this anyway.
        if re.match(r'0\\.|1\\.|6\\.0', pip_version):
          sys.stderr.write('The python_package resource requires pip >= 6.1.0, currently '+pip_version+'\\n')
          sys.exit(1)
        try:
          ModuleNotFoundError
        except NameError:
          ModuleNotFoundError = ImportError

        try:
          from pip.commands import InstallCommand
          from pip.index import PackageFinder
          from pip.req import InstallRequirement
        except ModuleNotFoundError:
          # Pip 19.x introduced an 'improved' pattern.
          from pip._internal.commands.install import InstallCommand
          try:
            from pip._internal.index.package_finder import PackageFinder
          except ModuleNotFoundError:
            # This occurs when the system pip==20.0.2 but you need to downgrade
            # virtualenv pip to 9.1.1
            from pip._internal.index import PackageFinder
        except ImportError:
          # Pip 10 moved all internals to their own package.
          from pip._internal.commands import InstallCommand
          from pip._internal.index import PackageFinder
          from pip._internal.req import InstallRequirement

        try:
          install_req_from_line = InstallRequirement.from_line
        except NameError:
          from pip._internal.req.constructors import install_req_from_line

        packages = {}
        try:
          cmd = InstallCommand()
        except TypeError:
          # Pip 20.x removed the default values, requiring explicit declarations
          cmd = InstallCommand(
                    name='install',
                    summary='Install packages.'
                )

        options, args = cmd.parse_args(sys.argv[1:])
        with cmd._build_session(options) as session:
          if options.no_index:
            index_urls = []
          else:
            index_urls = [options.index_url] + options.extra_index_urls
          finder_options = dict(
            trusted_hosts=options.trusted_hosts,
            session=session
          )
          if getattr(options, 'format_control', None):
            finder_options['format_control'] = options.format_control

          # Pip 19.2 changed how the PackageFinder class works
          try:
            from pip._internal.models.search_scope import SearchScope
            from pip._internal.models.target_python import TargetPython
            try:
              from pip._internal.index import CandidatePreferences
            except ImportError:
              # Pip 20.x pattern
              from pip._internal.index.package_finder import CandidatePreferences

            finder_options['target_python'] = TargetPython()
            finder_options['allow_yanked'] = False
            finder_options['candidate_prefs'] = CandidatePreferences(
              allow_all_prereleases=options.pre
            )
            finder_options['search_scope'] = SearchScope(
              find_links=options.find_links,
              index_urls=index_urls
            )
          except ImportError:
            finder_options['find_links'] = options.find_links
            finder_options['index_urls'] = index_urls
            finder_options['allow_all_prereleases'] = options.pre

          try:
            finder = PackageFinder(**finder_options)
          except TypeError:
            # Pip 20.x patterns
            from pip._internal.index.collector import LinkCollector
            del finder_options['trusted_hosts']
            finder_options['link_collector'] = LinkCollector(
              session=finder_options.pop('session'),
              search_scope=finder_options.pop('search_scope')
            )
            finder = PackageFinder(**finder_options)

          find_all = getattr(finder, 'find_all_candidates', getattr(finder, '_find_all_versions', None))
          for arg in args:
            req = install_req_from_line(arg)
            found = finder.find_requirement(req, True)
            all_candidates = find_all(req.name)
            try:
              # Pip 19.2 renamed "location" to "link"
              candidate = [c for c in all_candidates if c.link == found]
            except AttributeError:
              candidate = [c for c in all_candidates if c.location == found]
            if candidate:
              try:
                packages[candidate[0].project.lower()] = str(candidate[0].version)
              except AttributeError:
                # Pip 20.x changed the return values from the Candidates.
                packages[candidate[0].name.lower()] = str(candidate[0].version)
        json.dump(packages, sys.stdout)
      HACK
    end
  end
end
